#include <windows.h>

const char WindowsClassName[] = "View";

#define FILE_Menu 1001
#define OPTIONS_Menu 2001

void Draw_to_screen(HWND hwnd);
bool test1 = false;

//Global variables for draw to screen
HDC hdc;
HDC hdcMem;
HBITMAP bitmap;

// Step 4: the Window Procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
		//----starting application initializations--[start]--------
        case WM_CREATE:
        {
            HMENU hMenu, hSubMenu;

            hMenu = CreateMenu();

            hSubMenu = CreatePopupMenu();
            AppendMenu(hSubMenu, MF_STRING, FILE_Menu, "E&xit");
            AppendMenu(hMenu, MF_STRING | MF_POPUP, (UINT)hSubMenu, "&File");

            hSubMenu = CreatePopupMenu();
            AppendMenu(hSubMenu, MF_STRING, OPTIONS_Menu, "&Options");
            AppendMenu(hMenu, MF_STRING | MF_POPUP, (UINT)hSubMenu, "&Options");

            SetMenu(hwnd, hMenu);

        }
        break;
		//----starting application initializations--[end]-----------

		//----Menu options--------------------------[start]---------
        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case FILE_Menu:
					{
						PostMessage(hwnd, WM_CLOSE, 0, 0);
					}

                break;

                case OPTIONS_Menu:
					{
						test1 = true;
						
					}

                break;
            }
        break;
		//----Menu options--------------------------[end]-------------

	    case WM_PAINT:
		{
			//Begin Paint/ End Paint only applicable inside this WM_PAINT
	        PAINTSTRUCT ps;

	        HDC hdc = BeginPaint(hwnd, &ps);
		
			DeleteDC(hdc);

	        EndPaint(hwnd, &ps);
	    }
	    break;

        case WM_CLOSE:
            DestroyWindow(hwnd);
        break;

        case WM_DESTROY:
            PostQuitMessage(0);
        break;
        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    
	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{
    WNDCLASSEX wc;
    HWND hwnd;
    MSG Msg;

    //Step 1: Registering the Window Class
    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = 0;
    wc.lpfnWndProc   = WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = WindowsClassName;
    wc.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

    if(!RegisterClassEx(&wc))
    {
        MessageBox(NULL, "Window Registration Failed!", "Error!",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    // Step 2: Creating the Window
    hwnd = CreateWindowEx(
        WS_EX_CLIENTEDGE,
        WindowsClassName,
        "View",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, 300, 300,
        NULL, NULL, hInstance, NULL);

    if(hwnd == NULL)
    {
        MessageBox(NULL, "Window Creation Failed!", "Error!",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);

	//------Setting up the back buffering for paint [start]--------
	hdc = GetDC(hwnd); 
	hdcMem = CreateCompatibleDC(hdc); 
	// always create the bitmap for the memdc from the window dc
	bitmap = CreateCompatibleBitmap(hdc,300,300);
	SelectObject(hdcMem, bitmap);

	// only execute the code up to this point one time
	// that is, you only need to create the back buffer once
	// you can reuse it over and over again after that

	//------Setting up the back buffering for paint [end]-----------

	unsigned int timer = 0;
	// message loop
	while(TRUE)
	{
	    // Check to see if any messages are waiting in the queue
	    while(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
	    {
	        // Translate the message and dispatch it to WindowProc()
	        TranslateMessage(&Msg);
	        DispatchMessage(&Msg);
	    }

	    // If the message is WM_QUIT, exit the while loop
	    if(Msg.message == WM_QUIT)
	        break;

		
		if (GetTickCount() - timer >= 500)
		{
			timer = GetTickCount();
			Draw_to_screen(hwnd);
		}
	}

	//------Removing the back buffering for paint [start]--------
	
	// Clean up - only need to do this one time as well
	DeleteDC(hdcMem);
	DeleteObject(bitmap);
	ReleaseDC(hwnd, hdc);
	
	//------Removing the back buffering for paint [end]----------
    return Msg.wParam;
}

void Draw_to_screen(HWND hwnd)
{
	RECT rect = {0,0, 300, 300};
	FillRect(hdcMem, &rect, (HBRUSH)(WHITE_BRUSH));
	
	SetTextColor(hdcMem, RGB(255,0,255));
	TextOut(hdcMem, 0, 0, "TEST", 4);
	
	// draw on hdcMem
	// for example  ...

	Rectangle(hdcMem, 50, 50, 100, 100);

	// when finished drawing blit the hdcMem to the hdc
	BitBlt(hdc, 0, 0, 300, 300, hdcMem, 0, 0, SRCCOPY);
}

/* Notes on Double Buffering

  HDC hdc; -Main display Context

  HDC hdcMem; -A copy of the main Context hdc

  HBITMAP bitmap; -Memmory for hdc (specifies height x width)

  SelectObject(hdcMem, bitmap); -Sets hdcMem to current bitmap settings
*/

/* Notes on PeekMessage() VS GetMessage()

  PeekMessage() - continues whether messages exist or not
  GetMessage() - waits for a message before continuing

*/